@@ -145,20 +145,30 @@ pls_cv <- function(x, y, ncomp,
                    group = NULL,
                    retrieve = TRUE,
                    tune = TRUE,
-                   max_iter = 1, tol = 1e-6) {
+                   max_iter = 1, tol = 1e-6, 
+                   seed = NULL,
+                   modified = FALSE) {
   min_allowed <- (floor(min(ncol(x), nrow(x)) * p)) - 1
-
+  
   if (min_allowed < ncomp) {
     ncomp <- min_allowed
   }
+  
+  if (modified) {
+    algorithm <- "mpls"
+  } else {
+    algorithm <- "pls"
+  }
+  
   cv_samples <- sample_stratified(
     y = y,
     p = p,
     number = number,
     group = group,
-    replacement = FALSE
+    replacement = FALSE,
+    seed = seed
   )
-
+  
   if (method == "wapls" & retrieve & tune) {
     pls_c <- c(min_component, ncomp)
     seq_pls <- min(pls_c):max(pls_c)
@@ -174,7 +184,7 @@ pls_cv <- function(x, y, ncomp,
     search_grid <- matrix(0, 0, 0)
     fit_method <- method
   }
-
+  
   cv_results <- opls_cv_cpp(
     X = x,
     Y = y,
@@ -187,12 +197,13 @@ pls_cv <- function(x, y, ncomp,
     new_x = new_x,
     maxiter = max_iter,
     tol = tol,
-    wapls_grid = search_grid
+    wapls_grid = search_grid, 
+    algorithm = algorithm
   )
-
+  
   val <- NULL
   val$resamples <- cv_samples$hold_out
-
+  
   if (method == "pls") {
     val$cv_results <- data.table(
       npls = 1:ncomp,
@@ -203,7 +214,7 @@ pls_cv <- function(x, y, ncomp,
     )
     best_pls_c <- val$cv_results$npls[which(val$cv_results$rmse_cv == min(val$cv_results$rmse_cv))]
     val$best_pls_c <- best_pls_c
-
+    
     if (retrieve) {
       if (!is.null(weights)) {
         x <- sweep(x, 1, weights, "*")
@@ -216,7 +227,8 @@ pls_cv <- function(x, y, ncomp,
           scale = scale,
           ncomp = best_pls_c,
           maxiter = max_iter,
-          tol = tol
+          tol = tol, 
+          algorithm = algorithm
         )
       } else {
         val$models <- opls_get_basics(
@@ -225,12 +237,13 @@ pls_cv <- function(x, y, ncomp,
           scale = scale,
           ncomp = ncomp,
           maxiter = max_iter,
-          tol = tol
+          tol = tol, 
+          algorithm = algorithm
         )
       }
     }
   }
-
+  
   if (method == "wapls" & retrieve & !tune) {
     val$compweights <- cv_results$compweights
     val$cv_results <- data.table(
@@ -241,8 +254,8 @@ pls_cv <- function(x, y, ncomp,
       rmse_sd_cv = sd(cv_results$rmse_seg),
       r2_cv = mean(cv_results$rsq_seg)
     )
-
-
+    
+    
     if (!is.null(weights)) {
       x <- sweep(x, 1, weights, "*") ### MODIFIED
       y <- y * weights
@@ -253,40 +266,42 @@ pls_cv <- function(x, y, ncomp,
       scale = scale,
       ncomp = ncomp,
       maxiter = max_iter,
-      tol = tol
+      tol = tol, 
+      algorithm = algorithm
     )
   }
-
+  
   if (method == "wapls" & retrieve & tune) {
     val$cv_results <- data.table(search_grid,
-      rmse_cv = rowMeans(cv_results$rmse_seg),
-      st_rmse_cv = rowMeans(cv_results$st_rmse_seg),
-      rmse_sd_cv = get_col_sds(t(cv_results$rmse_seg)),
-      r2_cv = rowMeans(cv_results$rsq_seg)
+                                 rmse_cv = rowMeans(cv_results$rmse_seg),
+                                 st_rmse_cv = rowMeans(cv_results$st_rmse_seg),
+                                 rmse_sd_cv = get_col_sds(t(cv_results$rmse_seg)),
+                                 r2_cv = rowMeans(cv_results$rsq_seg)
     )
     opmls <- which.min(val$cv_results$rmse_cv)
     val$best_pls_c$min_component <- val$cv_results$min_component[opmls]
     val$best_pls_c$max_component <- val$cv_results$max_component[opmls]
     val$compweights <- cv_results$compweights
-
-
+    
+    
     if (!is.null(weights)) {
       x <- sweep(x, 1, weights, "*")
       y <- y * weights
     }
-
+    
     tmp_compweights <- val$compweights[val$best_pls_c$min_component:val$best_pls_c$max_component]
     val$compweights[] <- 0
     tmp_compweights <- tmp_compweights / sum(tmp_compweights)
     val$compweights[val$best_pls_c$min_component:val$best_pls_c$max_component] <- tmp_compweights
-
+    
     val$models <- opls_get_basics(
       X = x,
       Y = y,
       scale = scale,
       ncomp = ncomp,
       maxiter = max_iter,
-      tol = tol
+      tol = tol, 
+      algorithm = algorithm
     )
   }
   val
@@ -340,7 +355,7 @@ get_wapls_weights <- function(pls_model, original_x, type = "w1", new_x = NULL,
   max_component <- pls_c[[2]]
 
 
-  whgt <- get_pls_weights(
+  whgt <- get_local_pls_weights(
     projection_mat = pls_model$projection_mat,
     xloadings = pls_model$X_loadings,
     coefficients = pls_model$coefficients,
@@ -625,7 +640,7 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
                             tune = FALSE, number = 10, p = 0.75,
                             group = NULL, noise_variance = 0.001,
                             range_prediction_limits = TRUE,
-                            pls_max_iter = 1, pls_tol = 1e-6) {
+                            pls_max_iter = 1, pls_tol = 1e-6, modified = FALSE, seed = NULL) {
   if (is.null(weights)) {
     weights <- 1
   }
@@ -635,6 +650,12 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
     scale <- FALSE
   }
 
+  if (modified) {
+    algorithm <- "mpls"
+  } else {
+    algorithm <- "pls"
+  }
+    
 
   results <- cv_val <- pred <- NULL
   if (pred_method == "gpr") {
@@ -647,7 +668,8 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         p = p,
         number = number,
         group = group,
-        retrieve = "final_model"
+        retrieve = "final_model", 
+        seed = seed
       )
       fit <- cv_val$model
     } else {
@@ -688,7 +710,9 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         retrieve = TRUE,
         tune = tune,
         max_iter = pls_max_iter,
-        tol = pls_tol
+        tol = pls_tol, 
+        modified = modified, 
+        seed = seed
       )
       fit <- cv_val$models
       ncomp <- cv_val$best_pls_c
@@ -701,7 +725,8 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         scale = scale,
         ncomp = pls_c,
         maxiter = pls_max_iter,
-        tol = pls_tol
+        tol = pls_tol, 
+        algorithm = algorithm
       )
       ncomp <- pls_c
     }
@@ -732,7 +757,9 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         retrieve = TRUE,
         tune = tune,
         max_iter = pls_max_iter,
-        tol = pls_tol
+        tol = pls_tol, 
+        modified = modified, 
+        seed = seed
       )
 
       fit <- cv_val$models
@@ -759,7 +786,8 @@ fit_and_predict <- function(x, y, pred_method, scale = FALSE, weights = NULL,
         scale = scale,
         ncomp = pls_c[[2]],
         maxiter = pls_max_iter,
-        tol = pls_tol
+        tol = pls_tol, 
+        algorithm = algorithm
       )
 
       # compute weights for PLS components selected (from plsMin to plsMax)
@@ -811,7 +839,8 @@ gaussian_pr_cv <- function(x,
                            number = 10,
                            group = NULL,
                            noise_variance = 0.001,
-                           retrieve = c("final_model", "none")) {
+                           retrieve = c("final_model", "none"), 
+                           seed = NULL) {
 
   ## Create the resampling groups
   cv_samples <- sample_stratified(
@@ -819,7 +848,8 @@ gaussian_pr_cv <- function(x,
     p = p,
     number = number,
     group = group,
-    replacement = FALSE
+    replacement = FALSE, 
+    seed = seed
   )
 
   cv_results <- gaussian_process_cv(
